# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

# pylint: disable=missing-module-docstring

try:
    # pylint: disable=used-before-assignment
    _EmbeddedImporter
except NameError:
    from .importer import _EmbeddedImporter


# FIXME: EmbeddedCodeGenerator: start
class _EmbeddedImporterRestoreSys(_EmbeddedImporter):
    # pylint: disable=import-outside-toplevel
    import sys

    def __init__(self):
        super().__init__()
        self._restore_sys_modules = {}

    def add_module(self, fullname: str, is_package: bool, filename: str, source_text: str, source_hash: str) -> None:
        # pylint: disable=too-many-arguments

        # Remove any embedded module from sys.modules so that any import in this script gets the embedded implementation.  Add the
        # existing module definition to a set of modules to be restored when this embedded importer instance is not longer needed,
        super().add_module(fullname, is_package, filename, source_text, source_hash)
        sys_modules = self.sys.modules
        if fullname in sys_modules:
            if fullname not in self._restore_sys_modules:
                self._restore_sys_modules[fullname] = sys_modules[fullname]
            del sys_modules[fullname]

    def cleanup_importer(self):
        # Remove this embedded importer from sys.meta_path.  Leave other instances of the embedded importer in case we are in a
        # multi-level eval/exec scenario.
        sys_meta_path = self.sys.meta_path
        for idx, obj in enumerate(sys_meta_path):
            if self is obj:
                del sys_meta_path[idx]
                break

        # Remove any embedded modules from sys.modules that were added by this importer.
        sys_modules = self.sys.modules
        for fullname in self._registered_modules:
            if fullname in self._restore_sys_modules:
                sys_modules[fullname] = self._restore_sys_modules[fullname]
            else:
                if fullname in sys_modules:
                    del sys_modules[fullname]

        super().cleanup_importer()
