#!/usr/bin/env python3

# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import os
import sys
import argparse
import datetime
from dateutil.parser import parse as dp

parser = argparse.ArgumentParser(description="Reformat the timestamps in a file.")
parser.add_argument(
    "--format_from",
    dest="format_from",
    help=r"From Range format Ex: 2018-03-05 15:26:21 -> %%Y-%%m-%%d-%%H:%%M:%%S",
)
parser.add_argument(
    "--format_to",
    dest="format_to",
    help=r"To Range format Ex: 2018-03-05 15:26:21 -> %%Y-%%m-%%d %%H:%%M:%%S",
)
parser.add_argument("file_names", metavar="F", type=str, nargs="+", help="File Name ")


def format_time(file_name, format_from, format_to):
    """This will reformat the timestamps as it walks through the file.
    It prints inline while formatting to keep the file out of memory while looping."""
    now = datetime.datetime.now()
    now_str = now.strftime(format_from)
    format_length_from = len(now_str)
    now_str = now.strftime(format_to)
    format_length_to = len(now_str)

    with open(file_name, "r") as fd:
        line = fd.readline()
        pos = fd.tell()
        while line:
            line_ts_raw = line[:format_length_from]
            try:
                line_ts = dp(line_ts_raw)
            except ValueError:
                print(line.strip())
            else:
                print(line_ts.strftime(format_to), line[format_length_from:].strip())
            line = fd.readline()


def main(args):
    format_from = args.format_from
    format_to = args.format_to
    file_names = args.file_names
    for file_name in file_names:
        format_time(file_name, format_from, format_to)


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)
