# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import datetime

import pytest

from Octeres.SL.modules.ltm import get_Super_Location_to_Mask
from Octeres.bitmask import EnhancedBitmask, EnhancedBitmaskLike
from Octeres.bitmask.bitmask_lite import LiteBitmask

fixtures_bitmask_classes = (
    pytest.param(EnhancedBitmask),
    pytest.param(LiteBitmask),
)


class TestLTM:
    machine_name = "salag"

    @pytest.mark.parametrize("bitmask_class", fixtures_bitmask_classes)
    def test_00(self, bitmask_class: EnhancedBitmaskLike):
        ts = datetime.datetime(1970, 1, 1)
        te = datetime.datetime(2038, 1, 1)
        sltm = get_Super_Location_to_Mask(
            filt={"name": self.machine_name, "ts": ts, "te": te},
            bitmask_class=bitmask_class,
        )
        ltm_lst = sltm.ltm_objects_factory(self.machine_name, ts, te)
        for ltm_obj in ltm_lst:
            print(ltm_obj)
            assert len(ltm_obj.graph.nodes) > 0
